%% Curvature Analysis XTension (1 of 2)
%  
%  This is the first part of the curvature analysis. It:
%   -places a spot at each vertex on the surface of an Imaris Surface.
%   -calculates the mean, Gaussian and principal curvature values at each
%    of these spots.
%   -exports all of the calculations as a .mat file. 
%   -allows for the creation of a ROI. 
%
%   INPUTS: (1) A Surface in an Imaris Surpass Scene
%           (2) A ROI, constructed of spots on the surface of the Surface.
%               If no ROI is required, one single spot on the Surface
%               suffices.
%
%   OUTPUTS: (1) All calculations (including position coordinates) as a
%               .mat file
%            (2) A folder containing all of the spots on the surface,
%                exported into the current Surpass Scene in Imaris.
%                A folder containing the spots within the ROI, exported
%                into the current Surpass Scene in Imaris 
%
%  The core of this XTension was created by Matthew J Gastinger, PhD
%  (Bitplane) in 2014. It uses Dirk-Jan Kroon's “Patch
%  Curvature” function (available on MATLAB Central File Exchange).
%  
%  This XTension was edited and expanded by Maximilian S Jentzsch (Amy
%  Gladfelter lab, Dartmouth College, Hanover, NH, USA) in 2015.  
%
%  INSTALLTION:
% 
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Surfaces Functions">
%        <Item name="Curvature_MJG_MSJ" icon="Matlab">
%          <Command>MatlabXT::XT_MJG_MSJ_Curvature(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="Curvature_MJG_MSJ" icon="Matlab">
%            <Command>MatlabXT::XT_MJG_MSJ_Curvature(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 
%
%  Further Description (MJG):
%
%   This XTension only identifies those vertices with a normal z-vector <
%   -0.25. Thus only identifying the top surface.
% 
%  Further Description regarding the ROI creation (MSJ):
% 
%   The mechanism of ROI creation is somewhat simple; it uses the coordinates
%   of the manually created ROI spots as a filter. Multiple smaller
%   sub-ROIs may be required to create a satisfying ROI. This is best
%   done using the second XTension (XT_ROI_part2_MSJ.m) after the creation
%   of the first sub-ROI. 

%%
function XT_MJG_MSJ_Curvature(aImarisApplicationID)
tic

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
   javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

vProgressDisplay = waitbar(0, 'Measuring Curvature');

vFactory = vImarisApplication.GetFactory;


[pathstr, name, ext] = fileparts(char(vImarisApplication.GetCurrentFileName()));

% keyboard


vSelection = vImarisApplication.GetSurpassSelection;
if vFactory.IsDataContainer(vSelection)
    vParentsList = { vFactory.ToDataContainer(vSelection) };
elseif isempty(vSelection)
    vParentsList = { vImarisApplication.GetSurpassScene };
    if isequal(vParentsList, [])
        msgbox 'Please create a surpass scene!';
        return
    end
else
    vParentsList = { vSelection.GetParent };
end    

vParentsNumber = 1;

vSpotsList = {};
vSpotsNumber = 0;

vSurfacesList = {};
vSurfacesNumber = 0;

% get all the spots and surfaces (including sub directories)
  vParent = vParentsList{vParentsNumber};
  for vChild = 1:vParent.GetNumberOfChildren
    vItem = vParent.GetChild(vChild-1);
    if ~vItem.GetVisible
      continue
    end
    if vFactory.IsSpots(vItem)
      vSpotsNumber = vSpotsNumber + 1;
      vSpotsList{vSpotsNumber} = vFactory.ToSpots(vItem);
    elseif vFactory.IsSurfaces(vItem)
      vSurfacesNumber = vSurfacesNumber + 1;
      vSurfacesList{vSurfacesNumber} = vFactory.ToSurfaces(vItem);
    elseif vFactory.IsDataContainer(vItem)
      vParentsNumber = vParentsNumber + 1;
      vParentsList{vParentsNumber} = vFactory.ToDataContainer(vItem);
    end
  end


vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
  msgbox('Please create some Surfaces in the Surpass scene!');
  return;
end

% get the surfaces
vSurfaces = vImarisApplication.GetFactory.ToSurfaces(vImarisApplication.GetSurpassSelection);
if ~vImarisApplication.GetFactory.IsSurfaces(vSurfaces)  
  msgbox('Please select some Surfaces!');
  return;
end


if isempty(vSpotsList) == 1
        msgbox 'Please create some ROI spots!';
  return;
end


vROIcell = cell(1,vSpotsNumber);
for ROIspotsIndex = 1:vSpotsNumber
    
 
    vROIcoord = vSpotsList{ROIspotsIndex}.GetPositionsXYZ;
       
    xROImin = min(vROIcoord(:,1));
    xROImax = max(vROIcoord(:,1));
    yROImin = min(vROIcoord(:,2));
    yROImax = max(vROIcoord(:,2));
    zROImin = min(vROIcoord(:,3));
    zROImax = max(vROIcoord(:,3));
    
    ROImat = [ xROImin yROImin zROImin ; xROImax yROImax zROImax ];
    vROIcell{1,ROIspotsIndex} = ROImat;
    
    clear ROImat
    clear vROIcoord
    
end


% Create new group
vNormalsGroup = vImarisApplication.GetFactory.CreateDataContainer;
vNormalsGroup.SetName(['Spots on surface of ', char(vSurfaces.GetName)]);

% Add group
vSurpassScene.AddChild(vNormalsGroup, -1);

% Spots parameters
vNbPointsPerNormal = 1;
vPointRadius = 0.01;
vRed = 0;
vGreen = 0;
vBlue = 255;


vNumberOfSurfaces = vSurfaces.GetNumberOfSurfaces;


%%
for vSurfaceIndex = 0:vNumberOfSurfaces-1
	% create spot
    vSpots = vImarisApplication.GetFactory.CreateSpots;

    % get the vertices and normals
	vVertices = double(vSurfaces.GetVertices(vSurfaceIndex));
	% There is one normal per vertex
	vNormals = double(vSurfaces.GetNormals(vSurfaceIndex));
    vTriangles = double(vSurfaces.GetTriangles(vSurfaceIndex));
    
    
%%    
    FV=struct('vertices',vVertices,'faces',vTriangles);
    minvalues=min(FV.vertices);
    FV.vertices=bsxfun(@minus,FV.vertices,minvalues);
    %N=vNormals;
    FV.faces=FV.faces+1;
   
%%
    %Filter the normal Vectors
    vNormalsTopTest = vNormals(vNormals(:,3)<(-0.25));
    
    vTest=[vNormalsTopTest vNormalsTopTest vNormalsTopTest];
    vNormals(~vTest)=0;
    vNormals(all(vNormals==0,2),:)=[];
    vNormalsStats=vNormals;
    
    vVertices(~vTest)=0;
    vVertices(all(vVertices==0,2),:)=[];
    vVerticesFinal=vVertices;
    vNumberOfVertices = size(vVertices, 1);

    vSpotCountTop=size(vNormals,1);
    vTimeTop=zeros(vSpotCountTop,1);
    vRadiiTop=zeros(vSpotCountTop,1);
    vRadiiTop = vRadiiTop + 0.01;
    vNbPointsPerNormal=1;
    
	% Normalise the normal
	% Norme = sqrt(x^2 + y^2 + z^2)
	vNormalNorme = vNormals.^2;
	% Double transpose of the matrix to sum each row and no each column
	vNormalNorme = (sum(vNormalNorme'))';
	vNormalNorme = sqrt(vNormalNorme);
	% Divide each row of the matrix by the vector element at the same row
	vNormals = bsxfun(@rdivide, vNormals, vNormalNorme);
	
	% Duplicate each row on five rows
	% vNumberOfVertices is used because the number of vertices = the number of normals
	vNormals=vNormals(ceil((1:vNbPointsPerNormal*vSpotCountTop)/vNbPointsPerNormal), :);
	
	% Creation of a points sequence for one normal
	vPointsSequence = (0:vNbPointsPerNormal-1)';
	vPointsSequence = vPointsSequence * vPointRadius * 2;
	% Repeat this sequence for each normal
	vPointsSequence = repmat(vPointsSequence, vSpotCountTop, 1);
	vPointsSequencePos = bsxfun(@times, vNormals, vPointsSequence);
	
	% Duplicate each row on five rows
	vVertices=vVertices(ceil((1:vNbPointsPerNormal*vSpotCountTop)/vNbPointsPerNormal), :);
    vNormals = vPointsSequencePos + vVertices;
    
    vSpots.Set(vNormals, vTimeTop, vRadiiTop);
    vSpots.SetColorRGBA(vRed + vGreen*256 + vBlue*256*256);
    vSpots.SetName(['Spots on surface ',num2str(vSurfaceIndex+1)]);
    
%% This function calculates the principal curvature directions and values of a triangulated mesh. 
%
% The function first rotates the data so the normal of the current
% vertex becomes [-1 0 0], so we can describe the data by XY instead of
% XYZ.
% Secondly it fits a least-squares quadratic patch to the local 
% neighborhood of a vertex "f(x,y) = ax^2 + by^2 + cxy + dx + ey + f". 
% Then the eigenvectors and eigenvalues of the hessian are used to
% calculate the principal, mean and gaussian curvature.
%
% [Cmean,Cgaussian,Dir1,Dir2,Lambda1,Lambda2]=patchcurvature(FV,usethird) 
% Author: Dirk-Jan Kroon (part of Patch Curvature)

% Check inputs
if(nargin<2), usethird=true; end

% Number of vertices
nv=size(FV.vertices,1);

% Calculate vertices normals
N=patchnormals(FV);

% Calculate Rotation matrices for the normals
M= zeros(3,3,nv);
Minv= zeros(3,3,nv);
%vProgressDisplay2 = waitbar(0, 'Calculating Rotation matrices');
for i=1:nv, 
    [M(:,:,i),Minv(:,:,i)]=VectorRotationMatrix(N(i,:));
    %waitbar(i/nv);
end

%close(vProgressDisplay2);
% Get neighbours of all vertices
Ne=vertex_neighbours(FV);

% Loop through all vertices
Lambda1=zeros(nv,1);
Lambda2=zeros(nv,1);
Dir1=zeros(nv,3);
Dir2=zeros(nv,3);
%vProgressDisplay3 = waitbar(0, 'Calculating curvature');
for i=1:nv
   % Get first and second ring neighbours.
   if(~usethird)
       Nce=unique([Ne{Ne{i}}]);
   else
       % Get first, second and third ring neighbours
       Nce=unique([Ne{[Ne{Ne{i}}]}]);
   end
   
   Ve=FV.vertices(Nce,:);

   % Rotate to make normal [-1 0 0]
   We=double(Ve*Minv(:,:,i));
   f=We(:,1); x=We(:,2); y=We(:,3); 
   
   % Fit patch
   % f(x,y) = ax^2 + by^2 + cxy + dx + ey + f
   FM=double([x(:).^2 y(:).^2 x(:).*y(:) x(:) y(:) ones(numel(x),1)]);
   abcdef=FM\f(:);
%    keyboard
   a=abcdef(1); b=abcdef(2); c=abcdef(3);
   
   % Make Hessian matrix 
   % H =  [2*a c;c 2*b];
   Dxx = 2*a; Dxy=c; Dyy=2*b;
   
   [Lambda1(i),Lambda2(i),I1,I2]=eig2(Dxx,Dxy,Dyy);
   
   dir1=[0 I1(1) I1(2)]*M(:,:,i); 
   dir2=[0 I2(1) I2(2)]*M(:,:,i);
   Dir1(i,:)=dir1/sqrt(dir1(1)^2+dir1(2)^2+dir1(3)^2);
   Dir2(i,:)=dir2/sqrt(dir2(1)^2+dir2(2)^2+dir2(3)^2);
   %waitbar(i/nv);
end
%close(vProgressDisplay3);
%Curvature values for each vertex
cMean=(Lambda1+Lambda2)/2;
cGaussian=Lambda1.*Lambda2;



    %%
    %Create a new NormalVector Statistic
    vPointsCount = size(vNormalsStats,1);
    vNormalsStats=vNormalsStats(:);
    vStatsCount = numel(vNormalsStats);
    
    vNames = cell(vStatsCount, 1);
    vNames(1:vPointsCount) = {'Normal X'};
    vNames(vPointsCount+1:vPointsCount*2) = {'Normal Y'};
    vNames(vPointsCount*2+1:vStatsCount) = {'Normal Z'};
    vIds = [1:vPointsCount, 1:vPointsCount, 1:vPointsCount]' - 1;
    vUnits = cell(vStatsCount, 1);
    vUnits(:) = {'um'};
    vFactors = cell(3, vStatsCount);
    vFactors(1, :) = {'Spot'};
    vFactors(2, :) = {'1'};
    vFactors(3, :) = {'Normal Vector'};
    vFactorNames = {'Category','Time','Collection'};
    
    vSpots.AddStatistics(vNames, vNormalsStats, vUnits, vFactors, vFactorNames, vIds);
    
    % add custom statistics for mean curvature
    clear vInd    
    clear vUnits
    clear vFactors
    clear vFactorNames
    clear vNames
    
    vInd = 1:numel(cMean);
    vIds = vInd - 1;
    vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
    vFactors(vInd) = {'Spot'};
    vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
    vFactorNames = {'Category',};
    vNames(vInd) = {'Curvature mean'};
    

    vSpots.AddStatistics(vNames, cMean, vUnits, vFactors, vFactorNames, vIds);
    clear vInd    
    clear vUnits
    clear vFactors
    clear vFactorNames
    clear vNames
    % add custom statistics for Gaussian curvature

    vInd = 1:numel(cMean);
    vIds = vInd - 1;
    vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
    vFactors(vInd) = {'Spot'};
    vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
    vFactorNames = {'Category',};
    vNames(vInd) = {'Cuvature Gaussian'};
    
    
    vSpots.AddStatistics(vNames, cGaussian, vUnits, vFactors, vFactorNames, vIds);
    
    clear vInd    
    clear vUnits
    clear vFactors
    clear vFactorNames
    clear vNames
    % add custom statistics for principal curvatures

    vInd = 1:numel(cMean);
    vIds = vInd - 1;
    vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
    vFactors(vInd) = {'Spot'};
    vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
    vFactorNames = {'Category',};
    vNames(vInd) = {'Lambda1'};
    
    
    vSpots.AddStatistics(vNames, Lambda1, vUnits, vFactors, vFactorNames, vIds);
    
    clear vInd    
    clear vUnits
    clear vFactors
    clear vFactorNames
    clear vNames

    vInd = 1:numel(cMean);
    vIds = vInd - 1;
    vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
    vFactors(vInd) = {'Spot'};
    vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
    vFactorNames = {'Category',};
    vNames(vInd) = {'Lambda2'};
    
    
    vSpots.AddStatistics(vNames, Lambda2, vUnits, vFactors, vFactorNames, vIds);
    
    clear vInd    
    clear vUnits
    clear vFactors
    clear vFactorNames
    clear vNames

    vInd = 1:numel(cMean);
    vIds = vInd - 1;
    vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
    vFactors(vInd) = {'Spot'};
    vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
    vFactorNames = {'Category',};
    vNames(vInd) = {'vX'};
    
	vNormalsGroup.AddChild(vSpots, -1);
    
    %% Filtering the ROI statistics - MSJ
       
        outpath = uigetdir(pwd,'Please select the directory where you would like to save the big data set');
        outNames = {'Vertices', 'Normals', 'Lambda1', 'Lambda2', 'Mean Curvature', 'Gaussian Curvature'};
        outCell = cell(1,6);
        outCell = [outNames ; outCell];
        outCell{2,1} = vVertices;
        outCell{2,2} = vNormals;
        outCell{2,3} = Lambda1;
        outCell{2,4} = Lambda2;
        outCell{2,5} = cMean;
        outCell{2,6} = cGaussian;
        
        save([outpath '\' name '_' num2str(now) '_allData.mat'],'outCell')
        
        % Create new folder (group) for new spots
        vNormalsGroup = vImarisApplication.GetFactory.CreateDataContainer;
        vNormalsGroup.SetName(['ROI spots on surface of ', char(vSurfaces.GetName)]);

        % add group
        vSurpassScene.AddChild(vNormalsGroup, -1);

   
    for filterIndex = 1:vSpotsNumber
        
        % Create new ROI Spots    
        vRSpots = vImarisApplication.GetFactory.CreateSpots;
        
        ROImat = vROIcell{1,filterIndex};
        
        % Create a binary mask of the ROI 
        vxROImask = (vVertices(:,1)>=ROImat(1,1) & vVertices(:,1)<=ROImat(2,1));
        vyROImask = (vVertices(:,2)>=ROImat(1,2) & vVertices(:,2)<=ROImat(2,2));
        vzROImask = (vVertices(:,3)>=ROImat(1,3) & vVertices(:,3)<=ROImat(2,3));
        vROImask = vxROImask + vyROImask + vzROImask;
        vROImask = vROImask>=3;
        vROImask = [vROImask vROImask vROImask];

        % Filter the vertices
        vVerticesROI = vVertices(vROImask);
        vVerticesROI = reshape(vVerticesROI,(length(vVerticesROI)/3),3); 

        % Filter the normals
        vNormalsROI = vNormals(vROImask);
        vNormalsROI = reshape(vNormalsROI,(length(vNormalsROI)/3),3);

        % Filter the curvature statistics 
        cMeanROI = cMean(vROImask(:,1));
        cGaussianROI = cGaussian(vROImask(:,1));
        Lambda1ROI = Lambda1(vROImask(:,1));
        Lambda2ROI = Lambda2(vROImask(:,1));

     
        % Spots need to be assigned sizes and time points in Imaris 
        Rspotsizes = repmat(0.01,length(cMeanROI),1);
        spot_times = zeros(length(cMeanROI),1);

        % Create ROI spots 
        vRSpots.Set(vVerticesROI, spot_times, Rspotsizes);
        vRSpots.SetColorRGBA(vRed + vGreen*256 + vBlue*100*256);
        vRSpots.SetName(['ROI ' num2str(filterIndex) ' Spots on surface ',num2str(vSurfaceIndex+1)]);

        % Add curvature statistics
        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Mean Curvature']};

        vRSpots.AddStatistics(vNames, cMeanROI, vUnits, vFactors, vFactorNames, vIds);

        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Gaussian Curvature']};

        vRSpots.AddStatistics(vNames, cGaussianROI, vUnits, vFactors, vFactorNames, vIds);

        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI' num2str(filterIndex) ' Lambda1']};

        vRSpots.AddStatistics(vNames, Lambda1ROI, vUnits, vFactors, vFactorNames, vIds);

        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Lambda2']};

        vRSpots.AddStatistics(vNames, Lambda2ROI, vUnits, vFactors, vFactorNames, vIds);
        
        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Normal X']};

        vRSpots.AddStatistics(vNames, vNormalsROI(:,1), vUnits, vFactors, vFactorNames, vIds);
        
        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Normal Y']};

        vRSpots.AddStatistics(vNames, vNormalsROI(:,2), vUnits, vFactors, vFactorNames, vIds);
        
        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Normal Z']};

        vRSpots.AddStatistics(vNames, vNormalsROI(:,3), vUnits, vFactors, vFactorNames, vIds);

        vNormalsGroup.AddChild(vRSpots, -1);
        
        
        clear vRSpots vROImask vxROImask vyROImask vzROImask
    end
    waitbar(vSurfaceIndex/(vNumberOfSurfaces-1));
end
close(vProgressDisplay);
toc
timerVal=toc;
H=msgbox(num2str(timerVal),'Timer');
%make Imaris visible
%vImarisApplication.SetVisible(~vImarisApplication.GetVisible);



function [Lambda1,Lambda2,I1,I2]=eig2(Dxx,Dxy,Dyy)
% | Dxx  Dxy |
% |          |
% | Dxy  Dyy |
%
% example,
%   Dxx=round(rand*10); Dxy=round(rand*10); Dyy=round(rand*10); 
%   [a,b,c,d]=eig2(Dxx,Dxy,Dyy)
%   D = [a 0;0 b];
%   V = [c(:) d(:)];
%   check =  sum(abs(M*V(:,1) - D(1,1)*V(:,1))) + sum(abs(M*V(:,2) - D(2,2)*V(:,2))) ;
%
% Author: Dirk-Jan Kroon (part of Patch Curvature)

% Compute the eigenvectors 
tmp = sqrt((Dxx - Dyy).^2 + 4*Dxy.^2);
v2x = 2*Dxy; v2y = Dyy - Dxx + tmp;

% Normalize
mag = sqrt(v2x.^2 + v2y.^2); i = (mag ~= 0);
v2x(i) = v2x(i)./mag(i);
v2y(i) = v2y(i)./mag(i);

% The eigenvectors are orthogonal
v1x = -v2y; v1y = v2x;

% Compute the eigenvalues
mu1 = (0.5*(Dxx + Dyy + tmp));
mu2 = (0.5*(Dxx + Dyy - tmp));

% Sort eigen values by absolute value abs(Lambda1)<abs(Lambda2)
if(abs(mu1)<abs(mu2))
    Lambda1=mu1;
    Lambda2=mu2;
    I2=[v1x v1y];
    I1=[v2x v2y];
else
    Lambda1=mu2;
    Lambda2=mu1;
    I2=[v2x v2y];
    I1=[v1x v1y];
end

function N=patchnormals(FV)
% This function PATCHNORMALS calculates the normals of a triangulated
% mesh. PATCHNORMALS calls the patchnormal_double.c mex function which 
% first calculates the normals of all faces, and after that calculates 
% the vertice normals from the face normals weighted by the angles 
% of the faces.
% Author: Dirk-Jan Kroon (part of Patch Curvature)
[Nx,Ny,Nz]=patchnormals_double(double(FV.faces(:,1)),double(FV.faces(:,2)),double(FV.faces(:,3)),double(FV.vertices(:,1)),double(FV.vertices(:,2)),double(FV.vertices(:,3)));
N=zeros(length(Nx),3);
N(:,1)=Nx; N(:,2)=Ny; N(:,3)=Nz;



function [Nx,Ny,Nz]=patchnormals_double(Fa,Fb,Fc,Vx,Vy,Vz)
%
%  [Nx,Ny,Nz]=patchnormals_double(Fa,Fb,Fc,Vx,Vy,Vz)
%  Author: Dirk-Jan Kroon (part of Patch Curvature)

FV.vertices=zeros(length(Vx),3);
FV.vertices(:,1)=Vx;
FV.vertices(:,2)=Vy;
FV.vertices(:,3)=Vz;

% Get all edge vectors
e1=FV.vertices(Fa,:)-FV.vertices(Fb,:);
e2=FV.vertices(Fb,:)-FV.vertices(Fc,:);
e3=FV.vertices(Fc,:)-FV.vertices(Fa,:);

% Normalize edge vectors
e1_norm=e1./repmat(sqrt(e1(:,1).^2+e1(:,2).^2+e1(:,3).^2),1,3); 
e2_norm=e2./repmat(sqrt(e2(:,1).^2+e2(:,2).^2+e2(:,3).^2),1,3); 
e3_norm=e3./repmat(sqrt(e3(:,1).^2+e3(:,2).^2+e3(:,3).^2),1,3);

% Calculate Angle of face seen from vertices
Angle =  [acos(dot(e1_norm',-e3_norm'));acos(dot(e2_norm',-e1_norm'));acos(dot(e3_norm',-e2_norm'))]';

% Calculate normal of face
 Normal=cross(e1,e3);

% Calculate Vertice Normals 
VerticeNormals=zeros([size(FV.vertices,1) 3]);
for i=1:size(Fa,1),
    VerticeNormals(Fa(i),:)=VerticeNormals(Fa(i),:)+Normal(i,:)*Angle(i,1);
    VerticeNormals(Fb(i),:)=VerticeNormals(Fb(i),:)+Normal(i,:)*Angle(i,2);
    VerticeNormals(Fc(i),:)=VerticeNormals(Fc(i),:)+Normal(i,:)*Angle(i,3);
end

V_norm=sqrt(VerticeNormals(:,1).^2+VerticeNormals(:,2).^2+VerticeNormals(:,3).^2)+eps;
VerticeNormals=VerticeNormals./repmat(V_norm,1,3);
Nx=VerticeNormals(:,1);
Ny=VerticeNormals(:,2);
Nz=VerticeNormals(:,3);

    
    

function [M,Minv]=VectorRotationMatrix(v)
% [M,Minv]=VectorRotationMatrix(v,k)
% Author: Dirk-Jan Kroon (part of Patch Curvature)

v=(v(:)')/sqrt(sum(v.^2));
k=rand(1,3);
l = [k(2).*v(3)-k(3).*v(2), k(3).*v(1)-k(1).*v(3), k(1).*v(2)-k(2).*v(1)]; l=l/sqrt(sum(l.^2));
k = [l(2).*v(3)-l(3).*v(2), l(3).*v(1)-l(1).*v(3), l(1).*v(2)-l(2).*v(1)]; k=k/sqrt(sum(k.^2));
Minv=[v(:) l(:) k(:)];
% keyboard
M=inv(Minv);

function Ne=vertex_neighbours(FV)
% This function VERTEX_NEIGHBOURS will search in a face list for all 
% the neigbours of each vertex.
%
% Ne=vertex_neighbours(FV)
% Author: Dirk-Jan Kroon (part of Patch Curvature)
Ne=vertex_neighbours_double(FV.faces(:,1),FV.faces(:,2),FV.faces(:,3),FV.vertices(:,1),FV.vertices(:,2),FV.vertices(:,3));

function Ne=vertex_neighbours_double(Fa,Fb,Fc,Vx,Vy,Vz)
% Author: Dirk-Jan Kroon (part of Patch Curvature)
F=[Fa Fb Fc];
V=[Vx Vy Vz];

% Neighbourh cell array 
Ne=cell(1,size(V,1));

% Loop through all faces
for i=1:length(F)
    % Add the neighbors of each vertice of a face
    % to his neighbors list.
    Ne{F(i,1)}=[Ne{F(i,1)} [F(i,2) F(i,3)]];
    Ne{F(i,2)}=[Ne{F(i,2)} [F(i,3) F(i,1)]];
    Ne{F(i,3)}=[Ne{F(i,3)} [F(i,1) F(i,2)]];
end

% Loop through all neighbor arrays and sort them (Rotation same as faces)
for i=1:size(V,1)
 
    Pneighf=Ne{i};
    if(isempty(Pneighf))
        Pneig=[];
    else
        start=1;
        for index1=1:2:length(Pneighf)
            found=false;
            for index2=2:2:length(Pneighf),
                if(Pneighf(index1)==Pneighf(index2))
                    found=true; break
                end
            end
            if(~found)
                start=index1; break
            end
        end
        Pneig=[];
        Pneig(1)=Pneighf(start);
        Pneig(2)=Pneighf(start+1);
        
        % Add the neighbours with respect to original rotation
        for j=2+double(found):(length(Pneighf)/2)
            found = false;
            for index=1:2:length(Pneighf),
                if(Pneighf(index)==Pneig(end))
                    if(sum(Pneig==Pneighf(index+1))==0)
                        found =true;
                        Pneig=[Pneig Pneighf(index+1)];
                    end
                end
            end
            if(~found) % This only happens with weird edge vertices
                for index=1:2:length(Pneighf),
                    if(sum(Pneig==Pneighf(index))==0)
                        Pneig=[Pneig Pneighf(index)];
                        if(sum(Pneig==Pneighf(index+1))==0)
                            Pneig=[Pneig Pneighf(index+1)];
                        end
                    end
                end
            end
        end
        % Add forgotten neigbours
        if(length(Pneig)<length(Pneighf))
            for j=1:length(Pneighf)
                if(sum(Pneig==Pneighf(j))==0)
                    Pneig=[Pneig Pneighf(j)];
                end
            end
        end
    end
    Ne{i}=Pneig;
end  

