%% Curvature Analyis Xtension (2 of 2)
%
% This XTension was created to make the ROIing process faster. The way ROIs
% are created using XT_MJG_MSJ_Curvature is somewhat primitive, resulting
% in the need to create multiple smaller sub-ROIs to create the full ROI
% for the curvature analysis. Instead of having to run the full curvature
% calculation every single time a new smaller sub-ROI is added, this
% XTension takes the .mat file created from XT_MJG_MSJ_Curvature to create
% the newly added sub-ROI(s). Duplicate points need to be filtered out in a
% later analysis step. For the publication, the filtering was done in R.   
%
% Author: 
% Maximilian S. Jentzsch
% Amy Gladfelter Lab
% Dartmouth College, Hanover, NH, USA
%
% Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Surfaces Functions">
%        <Item name="ROI part 2 MSJ" icon="Matlab">
%          <Command>MatlabXT::XT_ROI_part2_MSJ(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%      <SurpassTab>
%        <SurpassComponent name="bpSurfaces">
%          <Item name="ROI part 2 MSJ" icon="Matlab">
%            <Command>MatlabXT::XT_ROI_part2_MSJ(%i)</Command>
%          </Item>
%        </SurpassComponent>
%      </SurpassTab>
%    </CustomTools>
% 

function XT_ROI_part2_MSJ(aImarisApplicationID)
%%
tic
% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
   javaaddpath ImarisLib.jar
  vImarisLib = ImarisLib;
  if ischar(aImarisApplicationID)
    aImarisApplicationID = round(str2double(aImarisApplicationID));
  end
  vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
  vImarisApplication = aImarisApplicationID;
end

vFactory = vImarisApplication.GetFactory;

vSelection = vImarisApplication.GetSurpassSelection;
if vFactory.IsDataContainer(vSelection)
    vParentsList = { vFactory.ToDataContainer(vSelection) };
elseif isempty(vSelection)
    vParentsList = { vImarisApplication.GetSurpassScene };
    if isequal(vParentsList, [])
        msgbox 'Please create a surpass scene!';
        return
    end
else
    vParentsList = { vSelection.GetParent };
end    

vParentsNumber = 1;

vSpotsList = {};
vSpotsNumber = 0;

vSurfacesList = {};
vSurfacesNumber = 0;

% get all the spots and surfaces (excluding subdirectories)
  vParent = vParentsList{vParentsNumber};
  for vChild = 1:vParent.GetNumberOfChildren
    vItem = vParent.GetChild(vChild-1);
    if ~vItem.GetVisible
      continue
    end
    if vFactory.IsSpots(vItem)
      vSpotsNumber = vSpotsNumber + 1;
      vSpotsList{vSpotsNumber} = vFactory.ToSpots(vItem);
    elseif vFactory.IsSurfaces(vItem)
      vSurfacesNumber = vSurfacesNumber + 1;
      vSurfacesList{vSurfacesNumber} = vFactory.ToSurfaces(vItem);
    end
  end

if isempty(vSpotsList) == 1
        msgbox 'Please create some ROI spots!';
  return;
end

vSurfaces = vImarisApplication.GetFactory.ToSurfaces(vImarisApplication.GetSurpassSelection);
if ~vImarisApplication.GetFactory.IsSurfaces(vSurfaces)  
  msgbox('Please select some Surfaces!');
  return;
end


display('Please select the file that contains all of the Vertices')
uiopen('matlab')

vVertices = outCell{2,1};
vNormals = outCell{2,2};
Lambda1 = outCell{2,3};
Lambda2 = outCell{2,4};
cMean = outCell{2,5};
cGaussian = outCell{2,6};

vROIcell = cell(1,vSpotsNumber);
for ROIspotsIndex = 1:vSpotsNumber
    
 
    vROIcoord = vSpotsList{ROIspotsIndex}.GetPositionsXYZ;
      
    xROImin = min(vROIcoord(:,1));
    xROImax = max(vROIcoord(:,1));
    yROImin = min(vROIcoord(:,2));
    yROImax = max(vROIcoord(:,2));
    zROImin = min(vROIcoord(:,3));
    zROImax = max(vROIcoord(:,3));
    
    ROImat = [ xROImin yROImin zROImin ; xROImax yROImax zROImax ];
    vROIcell{1,ROIspotsIndex} = ROImat;
    
    clear ROImat
    clear vROIcoord
    
end

%%
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
  msgbox('Please create some Surfaces in the Surpass scene!');
  return;
end

vSurfaceIndex = 0;
vNormalsGroup = vImarisApplication.GetFactory.CreateDataContainer;
vNormalsGroup.SetName(['ROI (2) spots on surface of ', char(vSurfaces.GetName)]);
vSurpassScene.AddChild(vNormalsGroup, -1);

for filterIndex = 1:vSpotsNumber
        
        % Create new ROI Spots    
        vRSpots = vImarisApplication.GetFactory.CreateSpots;
        
        ROImat = vROIcell{1,filterIndex};
        
        % Create a binary mask of the ROI 
        vxROImask = (vVertices(:,1)>=ROImat(1,1) & vVertices(:,1)<=ROImat(2,1));
        vyROImask = (vVertices(:,2)>=ROImat(1,2) & vVertices(:,2)<=ROImat(2,2));
        vzROImask = (vVertices(:,3)>=ROImat(1,3) & vVertices(:,3)<=ROImat(2,3));
        vROImask = vxROImask + vyROImask + vzROImask;
        vROImask = vROImask>=3;
        vROImask = [vROImask vROImask vROImask];

        % Filter the vertices
        vVerticesROI = vVertices(vROImask);
        vVerticesROI = reshape(vVerticesROI,(length(vVerticesROI)/3),3); 

        % Filter the normals
        vNormalsROI = vNormals(vROImask);
        vNormalsROI = reshape(vNormalsROI,(length(vNormalsROI)/3),3);

        % Filter the curvature statistics 
        cMeanROI = cMean(vROImask(:,1));
        cGaussianROI = cGaussian(vROImask(:,1));
        Lambda1ROI = Lambda1(vROImask(:,1));
        Lambda2ROI = Lambda2(vROImask(:,1));

      

        % Spots need to be assigned sizes and time points in Imaris 
        Rspotsizes = repmat(0.01,length(cMeanROI),1);
        spot_times = zeros(length(cMeanROI),1);

        % Create ROI spots 
     
        vRed = 0;
        vGreen = 0;
        vBlue = 255;
        vRSpots.Set(vVerticesROI, spot_times, Rspotsizes);
        vRSpots.SetColorRGBA(vRed + vGreen*256 + vBlue*100*256);
        vRSpots.SetName(['ROI ' num2str(filterIndex) ' Spots on surface ',num2str(vSurfaceIndex+1)]);

        % Add curvature statistics.
        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Mean Curvature']};

        vRSpots.AddStatistics(vNames, cMeanROI, vUnits, vFactors, vFactorNames, vIds);

        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Gaussian Curvature']};

        vRSpots.AddStatistics(vNames, cGaussianROI, vUnits, vFactors, vFactorNames, vIds);

        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI' num2str(filterIndex) ' Lambda1']};

        vRSpots.AddStatistics(vNames, Lambda1ROI, vUnits, vFactors, vFactorNames, vIds);

        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Lambda2']};

        vRSpots.AddStatistics(vNames, Lambda2ROI, vUnits, vFactors, vFactorNames, vIds);
        
        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Normal X']};

        vRSpots.AddStatistics(vNames, vNormalsROI(:,1), vUnits, vFactors, vFactorNames, vIds);
        
        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Normal Y']};

        vRSpots.AddStatistics(vNames, vNormalsROI(:,2), vUnits, vFactors, vFactorNames, vIds);
        
        clear vInd    
        clear vUnits
        clear vFactors
        clear vFactorNames
        clear vNames

        vInd = 1:numel(cMeanROI);
        vIds = vInd - 1;
        vUnits(vInd) = { char(vImarisApplication.GetDataSet.GetUnit) };
        vFactors(vInd) = {'Spot'};
        vFactors(vInd) = cellfun(@num2str, vFactors(vInd), 'UniformOutput', false);
        vFactorNames = {'Category',};
        vNames(vInd) = {['ROI ' num2str(filterIndex) ' Normal Z']};

        vRSpots.AddStatistics(vNames, vNormalsROI(:,3), vUnits, vFactors, vFactorNames, vIds);

        vNormalsGroup.AddChild(vRSpots, -1);
             
        clear vRSpots vROImask vxROImask vyROImask vzROImask
end
toc
end
