function filamentOrientation
%% filamentOrientation %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Reads in FIJI measurement xls files that contain filament angles.
% Calculates filament angles with respect to branch angle.
%
% INPUT: none. Just have all of your xls files in one folder, and know
% where that folder is located.
%   
% OUTPUT: a histogram, and a csv file with the final angles. The csv file
% is saved into a 'Results' folder in the same folder where the input files
% are located. 
% 
% Author:   Maximilian S. Jentzsch
%           Amy Gladfelter Lab 
%           Dartmouth College, Hanover, NH, USA
%           8/2015
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Get directory of where the .xls files are located
path = [uigetdir(pwd,'Where are the .xls files?') '/'];
xlsFiles = dir([path '*.xls']);

angles = [];

for i=1:length(xlsFiles)

    fname = xlsFiles(i).name;
        
    % Needs to be read in basic mode. You could change it to csv so that
    % it will be compatible on a variety of different machines. 
    data = xlsread([path fname]);
    if ispc == 1
        warning('Looks like you are using a Windows OS. Please check that you are reading the xls file in basic mode, or that the information is read in correctly. Thanks!')
    end
    
    % Get membrane angles
    MA = data(find(data(:,1)==1):find(data(:,1)==2),2); %had some issues with NaN while reading in xls files
    
    % Calculate branch orientation (relative to horizontal) by averaging
    % the two membrane angles. Also control for the scenario that the
    % membrane angle measurements are close to horizontal, e.g. one has an
    % angle of 2 degrees, and the other one of 355 degrees.
    
    lrg = max(MA);
    sml = min(MA);
    if abs(lrg-sml)>=290
        fromHoriz = lrg-360;
        if fromHoriz>0
            warning('Something went wrong! fromHoriz should be negative!')
        end
        Q = (fromHoriz+sml)/2;
        if Q<0
            MA_ave = 360+Q;
        else
            MA_ave = Q;
        end
    else
    MA_ave = mean(MA);
    end
    
    % get filament angles
    fil = data(find(data(:,1)==3):end,2);
    
    
    % subtract the average from the filament angles
    fil = fil - MA_ave;
    
    % Correct for inconsistent measurement direction. If you didn't measure
    % in the same direction as branch angles, the angle is offset by 180
    % degrees.
    fil(90<fil & fil<270) =   fil(90<fil & fil<270)-180;
    if any(fil<-90 & fil>-270)==1
        warning(['CAREFUL! Situation has occured that there are values between -90 and -270. Check ' fname ' and potentially debug!'])
    end
    
    % Convert remaining angles so that they represent the discrepancy from
    % the branch. 
    fil(fil>270 & fil<360) =   fil(fil>270 & fil<360)-360;
    fil(fil<-270 & fil>-360) =   fil(fil<-270 & fil>-360) + 360;
    
    
    angles = [angles; fil];
    
end

% Reorient angles so that to the left of branch orientation is negative,
% and to the right of branch orientation is positive.
angles = angles*(-1);
hist(angles)
length(angles)

% Export the results 
mkdir(path,'/Results/');
csvwrite([path '/Results/Results_angles.csv'], angles)

end

